@@ -19,34 +19,55 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <common.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+#include <time.h>
+#include <netinet/in.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/wait.h> /* POSIX.1 compliant */
 #ifdef __HAVE_DEBUG__
 #include <linux/if_ether.h>
 #endif
+#include <configuration.h>
+#include <t50_defines.h>
+#include <t50_typedefs.h>
+#include <t50_config.h>
+#include <t50_netio.h>
+#include <t50_errors.h>
+#include <t50_cidr.h>
+#include <t50_memalloc.h>
+#include <t50_modules.h>
+#include <t50_randomizer.h>
+#include <t50_shuffle.h>
 
 static pid_t pid = -1;      /* -1 is a trick used when __HAVE_TURBO__ isn't defined. */
 static sig_atomic_t child_is_dead = 0; /* Used to kill child process if necessary. */
 
 _NOINLINE static void               initialize(const struct config_options *);
 _NOINLINE static modules_table_t *  selectProtocol(const struct config_options * const, int *);
-_NOINLINE static const char *       get_ordinal_suffix(unsigned);
-_NOINLINE static const char *       get_month(unsigned);
+_NOINLINE static const char * const get_ordinal_suffix(unsigned int);
+_NOINLINE static const char * const get_month(unsigned int);
 
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-parameter"
 
 /**
- * Main function launches all T50 modules 
+ * Main function launches all T50 modules
  */
 int main(int argc, char *argv[])
 {
   struct config_options *co;
   struct cidr           *cidr_ptr;
   modules_table_t       *ptbl;
-  int                   proto; /* Used on main loop. */
+  int                   proto;
+  time_t                lt;
+  struct tm             *tm;
 
-  /* Parse_command_line returns ONLY if there are no errors. 
+  /* Parse_command_line returns ONLY if there are no errors.
      This must be called before testing user privileges. */
   co = parse_command_line(argv);
 
@@ -54,10 +75,7 @@ int main(int argc, char *argv[])
   if (getuid())
     fatal_error("User must have root privilege to run.");
 
-  /* General initializations. */
   initialize(co);
-
-  /* create_socket() handles its own errors before returning. */
   create_socket();
 
   /* Calculates CIDR for destination address. */
@@ -69,19 +87,17 @@ int main(int argc, char *argv[])
   if (co->turbo)
   {
     /* if it's necessary to fork a new process... */
-    if ((co->ip.protocol == IPPROTO_T50 && 
-         co->threshold > (threshold_t)get_number_of_registered_modules()) ||
-        (co->ip.protocol != IPPROTO_T50 && 
-         co->threshold > 1))
+    if ((co->ip.protocol == IPPROTO_T50 && co->threshold > number_of_modules) ||
+        (co->ip.protocol != IPPROTO_T50 && co->threshold > 1))
     {
       threshold_t new_threshold;
 
       if ((pid = fork()) == -1)
-        #ifdef __HAVE_DEBUG__
+#ifdef __HAVE_DEBUG__
         fatal_error("Error creating child process: \"%s\".\nExiting..", strerror(errno));
-        #else
+#else
         fatal_error("Error creating child process");
-        #endif
+#endif
 
       /* Divide the process iterations in main loop between both processes. */
       new_threshold = co->threshold / 2;
@@ -98,18 +114,15 @@ int main(int argc, char *argv[])
 
   /* Setting the priority to both parent and child process. */
   if (setpriority(PRIO_PROCESS, PRIO_PROCESS, -15)  == -1)
-  #ifdef __HAVE_DEBUG__
+#ifdef __HAVE_DEBUG__
     fatal_error("Error setting process priority: \"%s\".\nExiting..", strerror(errno));
-  #else
+#else
     fatal_error("Error setting process priority");
-  #endif
+#endif
 
   /* Show launch info only for parent process. */
   if (!IS_CHILD_PID(pid))
   {
-    time_t lt;
-    struct tm *tm;
-
     /* Getting the local time. */
     lt = time(NULL);
     tm = localtime(&lt);
@@ -124,29 +137,43 @@ int main(int argc, char *argv[])
            tm->tm_sec);
   }
 
+  // SRANDOM is here because each process has its own
+  // random seed. Notice this is called after fork().
   SRANDOM();
 
+  // Indices used for IPPROTO_T50 shuffling.
+  build_indices();
+
   /* Preallocate packet buffer. */
   alloc_packet(INITIAL_PACKET_SIZE);
 
   /* Selects the initial protocol to use. */
-  ptbl = selectProtocol(co, &proto);
+  if (co->ip.protocol != IPPROTO_T50)
+    ptbl = selectProtocol(co, &proto);
+  else
+  {
+    proto = co->ip.protocol;
+    shuffle(indices, number_of_modules);
+    ptbl = &mod_table[get_index(co)];
+  }
 
   /* MAIN LOOP */
+  // OBS: flood means non stop injection.
+  //      threshold is the number of packets to inject.
   while (co->flood || co->threshold)
   {
-    /* Holds the actual packet size after module function call. */
+    /* Will hold the actual packet size after module function call. */
     size_t size;
 
     /* Set the destination IP address to RANDOM IP address. */
     co->ip.daddr = cidr_ptr->__1st_addr;
     if (cidr_ptr->hostid)
-      co->ip.daddr += RANDOM() % cidr_ptr->hostid;  /* FIXME: Shouldn't be +1? */ 
-
-    /* We need the address in network order. */
+      // cidr_ptr->hostid has bit 0=0. The remainder is always less
+      // then the divisor, so we need to add 1.
+      co->ip.daddr += RANDOM() % (cidr_ptr->hostid + 1);
     co->ip.daddr = htonl(co->ip.daddr);
 
-    /* Calls the 'module' function to build the packet. */
+    /* Finally, calls the 'module' function to build the packet. */
     co->ip.protocol = ptbl->protocol_id;
     ptbl->func(co, &size);
 
@@ -154,24 +181,23 @@ int main(int argc, char *argv[])
     /* I'll use this to fine tune the alloc_packet() function, someday! */
     if (size > ETH_DATA_LEN)
       fprintf(stderr, "[DEBUG] Protocol %s packet size (%zu bytes) exceed max. Ethernet packet data length!\n",
-              ptbl->acronym, size);
+              ptbl->name, size);
 #endif
 
     /* Try to send the packet. */
     if (unlikely(!send_packet(packet, size, co)))
 #ifdef __HAVE_DEBUG__
-      error("Packet for protocol %s (%zu bytes long) not sent", ptbl->acronym, size);
-      /* continue trying to send other packets on debug mode! */
+      error("Packet for protocol %s (%zu bytes long) not sent", ptbl->name, size);
+    /* continue trying to send other packets on debug mode! */
 #else
       fatal_error("Unspecified error sending a packet");
 #endif
 
-    /* If protocol if 'T50', then get the next true protocol. */
+    /* If protocol is 'T50', then get the next true protocol. */
     if (proto == IPPROTO_T50)
-      if ((++ptbl)->func == NULL)
-        ptbl = mod_table;
+      ptbl = &mod_table[get_index(co)];
 
-    /* Decrement the threshold only if necessary! */
+    /* Decrement the threshold only if not flooding! */
     if (!co->flood)
       co->threshold--;
   }
@@ -179,25 +205,29 @@ int main(int argc, char *argv[])
   /* Show termination message only for parent process. */
   if (!IS_CHILD_PID(pid))
   {
-    time_t lt;
-    struct tm *tm;
-
+#ifdef __HAVE_TURBO__
+    // NOTE: Notice that for a single process pid will be -1! */
     if (pid > 0)
     {
+      // Don't do this if child process is already dead!
       if (!child_is_dead)
       {
         /* Wait 5 seconds for the child to end... */
         alarm(WAIT_FOR_CHILD_TIMEOUT);
 #ifdef __HAVE_DEBUG__
         fputs("\nWaiting for child process to end...\n", stderr);
-#endif        
-        if (wait(NULL) > 0)
-          child_is_dead = 1;
+#endif
+
+        /* SIGALRM will kill the child process if necessary! */
+        wait(NULL);
+        child_is_dead = 1;
+
         alarm(0);
       }
     }
+#endif
 
-    /* Finally we close the raw socket. */
+    /* Finally we close the raw socket. */ 
     close_socket();
 
     lt = time(NULL);
@@ -218,25 +248,23 @@ int main(int argc, char *argv[])
 }
 #pragma GCC diagnostic pop
 
-/* This function handles interruptions. */
+/* This function handles signal interrupts. */
 static void signal_handler(int signal)
 {
   /* NOTE: SIGALRM and SIGCHLD will happen only in parent process! */
-  if (signal == SIGALRM)
+  switch (signal)
   {
-    if (!IS_CHILD_PID(pid))
-      kill(pid, SIGKILL);
-    return;
+    case SIGALRM:
+      if (!IS_CHILD_PID(pid))
+        kill(pid, SIGKILL);
+      return;
+
+    case SIGCHLD:
+      child_is_dead = 1;
+      return;
   }
 
-  /* Child process terminated? */
-  if (signal == SIGCHLD)
-  {
-    child_is_dead = 1;
-    return;
-  }
-
-  close_socket();
+  close_socket();   // AS_SAFE!
 
   /* The shell documentation (bash) specifies that a process,
      when exits because a signal, must return 128+signal#. */
@@ -254,11 +282,11 @@ void initialize(const struct config_options *co)
 #ifndef __HAVE_DEBUG__
   sigaddset(&sigset, SIGTRAP);
 #endif
-  sigprocmask(SIG_BLOCK, &sigset, NULL); 
+  sigprocmask(SIG_BLOCK, &sigset, NULL);
 
   /* --- Initialize signal handlers --- */
   /* All these signals are handled by our handle. */
-  sigaction(SIGPIPE, &sa, NULL); 
+  sigaction(SIGPIPE, &sa, NULL);
   sigaction(SIGINT,  &sa, NULL);
   sigaction(SIGCHLD, &sa, NULL);
   sigaction(SIGALRM, &sa, NULL);
@@ -286,29 +314,24 @@ void initialize(const struct config_options *co)
 }
 
 /* Auxiliary function to return the [constant] ordinary suffix string for a number. */
-const char *get_ordinal_suffix(unsigned n)
+const char * const get_ordinal_suffix(unsigned int n)
 {
-  static const char *suffixes[] = { "st", "nd", "rd", "th" };
+  static const char * const suffixes[] = { "th", "st", "nd", "rd" };
 
-  /* FIX: 11, 12 & 13 have 'th' suffix, not 'st, nd or rd'. */
-  if ((n < 11) || (n > 13))
-    switch (n % 10)
-    {
-    case 1: return suffixes[0];
-    case 2: return suffixes[1];
-    case 3: return suffixes[2];
-    }
-
-  return suffixes[3];
+  if (n >= 11 && n <= 13) return suffixes[0];
+ 
+  // A little bit obfuscated, huh?
+  return suffixes["\000\001\002\003\000"
+                  "\000\000\000\000\000"[n % 10]];
 }
 
 /* Auxiliary function to return the [constant] string for a month.
-   NOTE: 'n' must be between 0 and 11. 
+   NOTE: 'n' must be between 0 and 11.
    NOTE: This routine is here just 'cause we need months in english. */
-const char *get_month(unsigned n)
+const char * const get_month(unsigned int n)
 {
   /* Months */
-  static const char *const months[] =
+  static const char * const months[] =
   {
     "Jan", "Feb", "Mar", "Apr", "May",  "Jun",
     "Jul", "Aug", "Sep", "Oct", "Nov",  "Dec"
